 
    function [Q,Qc,mQ,Qmins,v,ssort,isort]=getinitialQ(s,x, termcrit, sw,p,Qfixed,v)    
    % version: rmpc v3.0
    % Description: selects initial constraint set Q for algorithm rmpc
    %
    % inputs:
    %  s,x: iterate, 
    %  termcrit: current value of termination criterion
    %  sw,w: algorithm switches and parameters have instrunctions for how to select Q    
    %  Qfixed: a set that we always keep in Q (certain bound constraints for example)
    %  v: nx1 constraint valuation function discussed in paper.
    % 
    % outputs:
    %  Q,Qc: Q and Q complement indecies of constraints to be used and not
    %  mQ: size of Q
    %  Qmins: indicies of constraints in Q that are local minimizers of the "slack function" s
    %  v: updated nx1 constraint valuation function discussed in paper.
    %  ssort,isort: sorted version of s (largest to smallest) and the associated indexes so that ssort=s(isort)
    %  

     m=p.m;
     n=p.n;
     nn=(1:n)';
     
     Qmins=[];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    %%choose subset of constraints to process (playing around with various ideas here: work in progress)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %add M0 most active and fixed box/grid constraints
    [ssort, isort]=sort(s, 'ascend');
    Q=union(isort((1:p.M0)'), Qfixed);

%     %decluster the constraints: keep only minimum slack from each
%     %cluster of contiguous indicies. (decluster function is not efficient as currently written) 
%     %make sure to do this before adding the box constraints.
%     if(sw.Q_decluster)
%         Q=decluster(Q,s,m,n);
%     end

%     %add box constraints if requested
%     if(sw.box)
%         Q=union(Q,(1:2*m)');
%     end
  
    %cooling process for randomized constraints
    if(sw.Q_cool)
        Mrand=min(1,termcrit/p.Q_coolfactor)*p.Mrand;
    else
        Mrand=p.Mrand;
    end
    
   
    %give random and gridded constraints infinite value
    rp=randperm(n)';
    irand=rp(1:Mrand);

    %vforce holds indices of constraints we force into Q (could relax this later)
    vforce=zeros(n,1);
    vforce(irand)=inf;
   
    %give local mins infinite value
    if(sw.Q_addlocalmins)
        %lmtest1(i)&lmtest2(i)==1 if s(i) is a local 
        %minimizer of the slack function (based on its 2 immediate neighbors)
        lmtest1=s(1:n-2)>s(2:n-1);
        lmtest1=[0;lmtest1;0];
        lmtest2=s(2:n-1)<s(3:n);
        lmtest2=[0;lmtest2;0];
        %only keep those with corresponding slack value less than 1/2 maximum slack
        lmtest3=s<max(s((1+sw.box*2*m):n))/2; 
        %imins are the indecies of the local minimizers we add to Q
        imins=nn(lmtest1&lmtest2&lmtest3);
        %we will force these into Q 
        Qmins=imins;
        vforce(imins)=inf;
    end


    %valuate remaining constraints
    switch(sw.vadapt)
        case 'none'
            %do nothing
        case 'xos'
            %value constraints with large x_i/s_i
            xos = sqrt(x./s);
            v = 0*v + 1.0*p.etaQ*xos/max(xos);
        case 'oneos'
            %value constraints with small slack
            oneos = (1./s);
            v = 0*v + 1.0*p.etaQ*min(s)*oneos;
        otherwise
            error('invalid sw.vadapt\n');
    end

    %separated forced valuations and a filterable one
    iforce=find(vforce==inf);
    Mforce=length(iforce);
    if(Mforce<p.M1)
        Q=union(Q,iforce);
        M1prime=p.M1-Mforce; 
    else
        rp=randperm(Mforce);
        Q=union(Q,iforce(rp(1:p.M1)));
        M1prime=0;
    end
    
    %if adaptive rule is active    
    if(~strcmp(sw.vadapt, 'none')) 
        %sort value vector
        [vsort, isort] = sort(v, 'descend');

        %don't include any new constraints with value less than 1
        Mstop=min([find(vsort<1,1,'first')-1, M1prime]);

        %add additional constraints "adaptively" 
        Q=union(Q,isort(1:Mstop));
    end

    Qc=setdiff(nn,Q);
    mQ=length(Q);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
