function [A,b,c,y0,s0, H, g, gclean]=makechebprob(M,N, basis, fin, sigman, box)
% version: rmpc v3.0

%Luke Winternitz
%lukewinternitz@gmail.com
%April 2007

%Are rPDAS/rMPC good for chebyshev approximation problems?
%solve minimize ||Hy-g||_inf
%equivalently, 
%min t 
%s.t. Hy-g<=t 
%     g-Hy<=t
%in (y,t)
%
%H NxM


%inputs are size of the approximation problem size (m,n)
%M is number of basis functions and N is the length of the 
%reference signal
%
%the resulting LP will have A matrix of size m=M+1 x n=2*N
%the output is data (A,b,c) for an LP in standard form.
%min c'x s.t. Ax=b, x>=0 
%max b'y s.t. A'y+s=c, s>=0

%a dual strictly feasible point is returned that can be used as an
%initial guess (y0,s0)

%H0 is the approximation basis matrix so upon solving the LP 
%H0*yopt is the best approximation to the reference function g
%g is also returned



%basis='trig'

if(mod(M,2)==0)
    error('M must be odd for chebyshev problem');
end


%find_center=1;

NN=(0:N-1)';

Mo2=floor(M/2);
Hc=zeros(N,Mo2+1);
Hs=zeros(N,Mo2);
%H=zeros(N,M);

switch basis
    case 'trig'
        %trigonometric basis functions:
        %here we include j=0
        
        %this is good
        tic
        j=sqrt(-1);
        p=N;q=M;
        qo2=floor(q/2);
        pp=(0:N-1)';
        qq=1:qo2;                                   %q should be odd
        Hc=exp(j*2*pi/p*pp*qq)*sqrt(2/p);           %create complex basis matrix
        H=[ones(p,1)/sqrt(p), real(Hc), imag(Hc)]; %rearrange columns
        toc
        
        %this is not exactly equivalent but can be made to work and is
%         %fastest of the three
%         tic
%         E=speye(p);
%         Hc=ifft(full(E(:,1:qo2+1)))*sqrt(p);
%         H2=[real(Hc),imag(Hc(:,2:qo2+1))];
%         toc
        
%         %this is slow I expect 
%         tic
%         Hc(:,1)=ones(N,1)/sqrt(N);
%         for(j=2:Mo2+1)
%             Hc(:,j)=cos(2*pi*((j-1)/N)*NN)*sqrt(2/N);
%         end
% 
%         for(j=1:Mo2)
%             Hs(:,j)=sin(2*pi*(j/N)*NN)*sqrt(2/N);
%         end
% 
%         H=[Hc,Hs];
%         toc

    case 'legendre'
        
        tt=linspace(-1,1,N)';
        Hu=legendre(M-1, tt, 'norm')';
        H=Hu*diag(1./sqrt(sum(Hu.^2,1)));
    
    otherwise
        error('invalid basis')
end

        


%return this so that after solving can just compute H*y as approx
%since y has an extra entry
%H0=[H,zeros(n,1)]; 
    
    
%reference signal feel free to change.
%sigman=0.0;%noise standard deviation
f0=inline('sin(10*t).*cos((f*t).^2)', 't', 'f');
f_target=@(x) f0(x,fin);
tsamp=linspace(0,1,N)';
gclean=f_target(tsamp);


g=gclean+sigman*randn(N,1);

%in standard dual form

e=ones(N,1);
At=[H, -e; -H, -e];
c =[g;-g];
A=sparse(At');

%A is 2N x M+1
[m,n]=size(A);

b=[sparse(M,1);-1];


%if(find_center)
%    [y0,s0]=analytic_center(A,c);
%end

%this has an unbounded feasible region
%but nonetheless, a dual strictly feasible point is easy to find
y0=[sparse(M,1); max(abs(g)) + 1];
s0=c-A'*y0;


if(box>0)
    A=[speye(m),-speye(m),A];
    c=[box*ones(2*m,1);c];
    s0=c-A'*y0;
    if(min(s0)<0.9)
        error('increase size of bounding box');
    end
end



return





