function [A,b,c,y0,s0]=makerandLP(m, n, initpt, spar, box)
%version: rmpc v3.0
%function [A,b,c,y0,s0]=makerandLP(m, n, initpt, spar, box)

%inputs:
%mxn is requested size of A 
%initpt='nice' gives well centered initial point 
%initpt='poor' gives poorly centered initial point (by skewing feasible region really)
%spar in (0,1] is requested sparsity of A
%box nonzero adds as first 2m constraints, box constraints of the form: -box*e<=y<=box*e
%outputs:
%A,b,c: LP problem in std form
%y0,s0: dual feasible starting point.

showplots=1;

%keep same size of A
if(box>0) n=n-2*m; end

%spar is requested sparsity
if(spar==1)
    A=randn(m,n);
else
    A=sprandn(m,n,spar);
end

%%normalize columns after squeezing out zero columns this will change the size
%%just dont ask for too much sparsity if you want to aviod this.
%nA=sqrt(sum(A.^2,1))';
%A(:,nA==0)=[];
%[m,n]=size(A);
%
%normalize columns of A
%A(m,:)=ones(1,n);
nA=sqrt(sum(A.^2,1))';
if(any(nA==0))
    A(m,:)=ones(1,n);
    nA=sqrt(sum(A.^2,1))';
end
    
A=A*spdiags(1./nA,0,n,n);
b=randn(m, 1);

    
%provide well centered or poorly centered intial iterate. 
%really this affects the skewness of the feasible region
switch initpt
    case 'poor'
        nmid=ceil(n/2);
        s0=zeros(n,1);
        s0(1:nmid) = rand(nmid,1);
        s0(nmid+1:end) = 1e-4*rand(n-nmid,1);
    case 'nice'
        s0=rand(n,1)+ones(n,1);
    otherwise
        error('invalid init input')
end

%choose y0 randomly force feasibility by choice of c
y0 = randn(m,1);
c = A'*y0+s0;

%add bounding box constraints if requested 
if(box>0)
    A=[speye(m),-speye(m),A];
    c=[box*ones(2*m,1);c];
    s0=c-A'*y0;
    n=n+2*m;
end


%this is fun for 2/3 dimensional dual regions, 
%but requires (at least sufficient to have) 
%Multi-Parametric Toolbox (free online: google it)
if(showplots)
    if(m==2)        
        F=polytope(A',c);
        plot(F)
        hold on;
        plot(y0(1),y0(2), 'g*');
        %quiver(3,-2,b(1),b(2), 1);    
    elseif(m==3)        
        F=polytope(A',c);
        plot(F)
        hold on;
        plot3(y0(1),y0(2),y0(3), 'g*');
        quiver3(3,3,3, b(1),b(2),b(3),  1);

    end
end

    
    
