%----------------------------EXAMPLE OF HOW TO RUN RMPC---------------------------------
% version: rmpc v3.0
clear swin pin;

%no box constraints in dual
swin.box=0;
box=0;

problemtype=0; %0 for random, 1 for chebyshev

if(problemtype==0)
    %%%%%%%%%%%%% Generate a random LP %%%%%%%%%%%%%%%%%%%%
    m=200;
    n=40000;

    p_spar=1; %sparsity (dense if =1)

    [A,b,c,pin.y0,pin.s0]=makerandLP(m,n,'nice',p_spar,box*swin.box);
    if(p_spar>0.1)
        A=full(A);
    end

    %%%%%%%%%% set up parameters for rmpc %%%%%%%%%%%%%%
    pin.maxiter=300;
    pin.tol_conv=1e-8;
    swin.disp=1; %display iteration data
    swin.plotslacks=1 %plot slack function

    %constraint selection options
    pin.M=n; %upper bound on size of constraint set
    pin.M0=2*m; %number of most active constraints to keep
    pin.Mrand=0;%number of randomly selected constraints to keep
    swin.Q_cool=0; %whether to gradually reduce to zero the number of randomly selected constraints
    pin.Q_coolfactor=1.0;%approximate threshold value of termcrit where cooling begins
    pin.Mgrid=0; %number of gridded constraints to keep (approximately)
    swin.vadapt='none'; %adaptive constraint selection rule
    swin.Q_addlocalmins=0; %add local minimizers of slack function (appropriate for smooth chebyshev problem)

    dstar=0; %optimal dual value if you know it

else
    %%%%%%%%%%%%% Generate a chebyshev approximation LP %%%%%%%%%%%%%%%%%%%%
    %for chebyshev problem the "problem size" is smaller than the
    %resulting LP size.
    %specifically if min ||Hz-g||_inf is the goal and H is qxp (q>>p, p odd)
    %then the LP will be of size 2q x (p+1), i.e. n=2q, m=p+1
    %so given m,n for the LP size we
    %1) should choose m even so p=m-1 will be odd
    %2) take n even as well since q=n/2 should be whole

    M=199;
    N=(M+1)^2/2;

    basis='trig';
    %basis='legendre';
    sigman=0.0;

    box=1e3;
    swin.box=0;

    %corresponding LP size
    m=M+1;
    n=2*N;

    prob=sprintf('chebyshev-%dx%d',m,n);
    fin=5;
    [A,b,c,pin.y0,pin.s0, H, g,gclean]=makechebprob(M,N,basis,fin,sigman, box*swin.box);
    y0=pin.y0;
    s0=pin.s0;
    A=full(A);

    %%%%%%%%%% set up parameters rmpc %%%%%%%%%%%%%%
    pin.maxiter=300;
    pin.tol_conv=1e-8;
    swin.disp=1; %display iteration data
    swin.plotslacks=1 %plot slack function

    %constraint selection options
    pin.M=n; %upper bound on size of constraint set
    pin.M0=m; %number of most active constraints to keep
    pin.Mrand=0;%number of randomly selected constraints to keep
    swin.Q_cool=0; %whether to gradually reduce to zero the number of randomly selected constraints
    pin.Q_coolfactor=1.0;%approximate threshold value of termcrit where cooling begins
    pin.Mgrid=2*m; %number of gridded constraints to keep (approximately)
    swin.vadapt='none'; %adaptive constraint selection rule
    swin.Q_addlocalmins=1; %add local minimizers of slack function (appropriate for smooth chebyshev problem)

    dstar=0; %optimal dual value if you know it

end


%run it
[x,y,s,sw,p,t,h]=lp_rmpc(A, b, c, swin, pin);



