function [p,sw]=rmpc_get_defaults(m,n)

% version: rmpc v3.0
% Description: 
% set default values for various parameters (in structure p) and switches (in structure sw)
%
% Luke Winternitz
% Sept. 2007
% lukewinternitz at gmail dot com
%
% Inputs: 
% [m,n] = size of A matrix for the LP
%
%--------------------------problem parameters-----------------------------------

p.maxiter=300;          %maximum number of iterations.
p.tol_conv=1e-8;        %convergence tolerance
p.xlb=1e-10;             %maximum value of "lower bound" for primal variable
p.xub=1e30;             %upper bound for primal variable
p.lbexp=3;              %exponent appearing in lower bound |dyaff|^p.lbexp + |tildexaff_-|^p.lpexp
p.lbmult=1.0;           %multiplier for same lower bound term

p.beta=0.95;            %to maintain strict feasibility take only this fraction of step to boundary
p.theta_ascent=0.1;    %forces dual objective ascent
p.theta_stepsize=0.3;  %forces MPC step to be greater than than fraction of aff step

p.R1=1e9;               %safeguard parameters 
p.R2=1e9;               
p.R3=1e9;               
p.R4=1e9;               

%typically x0=p.xi*ones(n,1);
p.xi=1;
%placeholders to be replaced later
p.y0=sparse(m,1);
p.s0=ones(n,1);

p.dstar=0; %dual optimal value if known
p.z0=inf; % upper bound on dstar

%---------------------------------Select Algorithm---------------------------------

sw.disp=1;              %1-display info at each iteration, 0-dont


sw.alg = 'mpc++';   %'pdas: pdas, 
                    %'mpc': mpc without ascent and safeguards 
                    %'mpc+': mpc with gamma enforcing ascent,
                    %'mpc++': mpc with gamma enforcing ascent and dy^a->0 implies ccrhs -> 0   


%---------------------------how to choose intial point--------------------------------------------------

sw.initial_pdpoint = 'input';   %'input':use provided guess  
                                %'analytic center':use analytic center from SDPT3
                                %'mehrotra'2:use Mehrotra's heuristic based on ls soln to A'y=c and ensuring x,s>0

%-------------------------how to solve normal equations--------------------------------------
p.modchol_delta=1e-4;
p.modchol_zerothresh=1e-4;

sw.chol='chol_iter'; 
%---------------------------how to choose working set--------------------------------------------------


sw.Q_cool = 0;          %for random Q_rule whether to "cool" the randomness 
p.Q_coolfactor=1e-8;    %cooling begins approximately when termcrit=p.Q_coolfactor

%if the constraints depend continuously on a parameter it may be a good idea to include the 
%local maximizers of the slack-vector the the working constraint set.

sw.Q_addlocalmins = false;   
sw.Q_decluster=0;
                        

%constraint selection parameters and switches

p.M0=n;             %number of "most active" constraints to keep 
sw.box=0;           %if sw.box=1 first 2m constraints are of the form -pi e <= y <= pi e 
                        %for some big pi and these are kept in Q at all iterations (to eliminate degeneracy in A_Q)
p.Mgrid=0;          %number of gridded constraints (approx)
p.Mrand=0;          %number of random constraints
p.M=n;              %total number (or upper bound on) of constraints to keep
sw.vadapt='none';   %adaptive rule to use if any
p.etaQ=1e12;        %eta parameter for adaptive rule
p.M1=p.M-p.M0-2*m*sw.box-p.Mgrid;   %number of "additional" constraints to keep


%----------------------------more options-----------------------------------
sw.common_pdstep = false;   %false:different step length for primal and dual variables, 
                            %true:common step length for all variables                        
	
sw.soft_constr = true;  %false: no soft constraints                      
                        %true: constraints are soft, set s=max(tol_soft_constr, s);
p.tol_soft_constr=1e-20;                        
                        
sw.computeMrank=false; %compute rank of matrix M on each iteration uses a full SVD of M
sw.plotslacks=false; %plot slack variables on each iteration

%attempt to project iterate onto optimal face ala Kaliski/Ye
sw.attempt_proj=false;
p.proj_thresh=1e-3;

%safeguard switches
sw.safeguard='cont'; %'cont' or 'discont' 
sw.safeguard_redols=1; %if 'cont' redo the linesearch?



